unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, RegExpr, ExtCtrls, EditPlugAPI, EditPlugAPITypes,
  EditPlugFindUtils;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    bFind: TButton;
    cbWrap: TCheckBox;
    Panel2: TPanel;
    Label1: TLabel;
    Panel3: TPanel;
    edRepl: TMemo;
    cbRegex: TCheckBox;
    Panel4: TPanel;
    Label2: TLabel;
    edSearch: TMemo;
    bRepl: TButton;
    bReplAll: TButton;
    GroupBox: TGroupBox;
    rbCurrentFile: TRadioButton;
    rbAllOpenFiles: TRadioButton;
    Button1: TButton;
    procedure bFindClick(Sender: TObject);
    procedure bReplClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bReplAllClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    savedselstart: integer;
    savedsellen: integer;
    function FindRegex(inputstring: string; ShowNoMatch: boolean; idx: longint): boolean;
    function Find(inputstring: string; ShowNoMatch: boolean; idx: longint): boolean;
    function ExecFind(ShowNoMatch: boolean; edidx: integer): boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  gPlugId: HINST;

implementation

{$R *.DFM}


var
  r: TRegexpr;
  
{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    epSetParentToMainWin(form1.handle);
    form1.show;
    epMoveWinToTopRight(form1);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    form1.Free;
    form1:=nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;

{------------------------------------------------------------------------------}

procedure NoMatch(idx: integer);
begin
  showmessage('Could not find match');
  epEdRemoveSel(idx);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  savedselstart := 0;
  savedsellen := 0;
end;

// returns false if no match
function TForm1.FindRegex(inputstring: string; ShowNoMatch: boolean; idx: longint): boolean;
var
  endpos: integer;
  repl: string;
begin
  result := true;
  if edSearch.text = '' then exit;
  try
    r.Expression := ProcessEscapeChars(edSearch.text);
    r.InputString := inputstring;
    if r.ExecPos(epGetEdSelEnd(idx)+1) then begin
      savedselstart := r.matchpos[0]-1;
      epSetEdSelStart(idx, savedselstart);
      savedsellen := r.MatchLen[0];
      endpos:= r.matchpos[0] + savedsellen -1;
      epSetEdSelEnd(idx, endpos);
    end else begin
      if cbWrap.checked then begin
        if r.Exec then begin
          savedselstart := r.matchpos[0]-1;
          epSetEdSelStart(idx, savedselstart);
          savedsellen := r.MatchLen[0];
          endpos:= r.matchpos[0] + savedsellen -1;
          epSetEdSelEnd(idx, endpos);
        end else begin
          result := false;
          if ShowNoMatch then NoMatch(idx);
        end;
      end else begin
        result := false;
        if ShowNoMatch then NoMatch(idx);
      end;
    end;
  finally

  end;
end;

// returns false if no match
function TForm1.Find(inputstring: string; ShowNoMatch: boolean; idx: longint): boolean;
var
  endpos: integer;
  position: dword;
begin
  result := true;
  position := NextPos(edSearch.text, inputstring, epGetEdSelEnd(idx)+1);
  if position > 0 then begin
    savedselstart := position -1;
    epSetEdSelStart(idx, savedselstart);
    savedsellen := length(edSearch.text);
    endpos:= position + length(edSearch.text) -1;
    epSetEdSelEnd(idx, endpos);
  end else begin
    if cbWrap.checked then begin
      position := NextPos(edSearch.text, inputstring);
      if position > 0 then begin
        savedselstart := position -1;
        epSetEdSelStart(idx, savedselstart);
        savedsellen := length(edSearch.text);
        endpos:= position + savedsellen -1;
        epSetEdSelEnd(idx, endpos);
      end else begin
        result := false;
        if ShowNoMatch then NoMatch(idx);
      end;
    end else begin
      result := false;
      if ShowNoMatch then NoMatch(idx);
    end;
  end;
end;

// returns false if no match
function TForm1.ExecFind(ShowNoMatch: boolean; edidx: integer): boolean;
var inputstring: string;
begin
  result := true;
  savedsellen := 0;
  savedselstart := 0;
  if edSearch.text = '' then begin
    result := false;
    showmessage('Enter a search phrase');
    exit;
  end;
  inputstring := epGetEdTextAsString(edidx);
  if inputstring = '' then begin
    result := false;
    showmessage('No input string available to search');
    exit;
  end;
  if cbRegex.checked then
    result := FindRegex(inputstring, ShowNoMatch, edidx)
  else
    result := Find(inputstring, ShowNoMatch, edidx);
end;

procedure TForm1.bFindClick(Sender: TObject);
begin
  ExecFind(true, epGetEdIndex);
end;

procedure TForm1.bReplClick(Sender: TObject);
var
  selstart: integer;
  sellen: integer;
  repl: string;
begin
  selstart := epGetActiveEdSelStart;
  sellen := epGetActiveEdSelLen;
  if (selstart = savedselstart) and (sellen = savedsellen) and (sellen > 0) then begin
    if cbRegex.Checked then begin
      repl := r.substitute(edRepl.text);
      //showmessage(repl);
      epSetActiveEdSelText(pchar(repl));
    end else begin
      repl := edRepl.text;
      epSetActiveEdSelText(pchar(repl));
    end;
  end;
  bFindClick(nil);
end;

procedure TForm1.bReplAllClick(Sender: TObject);
var
  selstart: integer;
  sellen: integer;
  repl: string;
  i: integer;
  found: boolean;
  oldwrap: boolean;
begin
  if rbCurrentFile.Checked then begin
    repeat
      selstart := epGetActiveEdSelStart;
      sellen := epGetActiveEdSelLen;
      if (selstart = savedselstart) and (sellen = savedsellen) and (sellen > 0) then begin
        if cbRegex.Checked then begin
          repl := r.substitute(edRepl.text);
          epSetActiveEdSelText(pchar(repl));
        end else begin
          repl := edRepl.text;
          epSetActiveEdSelText(pchar(repl));
        end;
      end;
      found := ExecFind(false, epGetEdIndex);
    until not found;
  end;

  if rbAllOpenFiles.Checked then begin
    // when replacing in all files, wrap must be enabled
    oldwrap := cbWrap.checked;
    cbWrap.checked := true;
    for i := 0 to epGetEdCount -1 do begin
      repeat
        found := ExecFind(false, i);
        if found then begin
          if cbRegex.Checked then begin
            repl := r.substitute(edRepl.text);
            epSetEdSelText(i, pchar(repl));
          end else begin
            repl := edRepl.text;
            epSetEdSelText(i, pchar(repl));
          end;
        end;
      until not found;
    end;
   cbWrap.checked:= oldwrap;
  end;

end;

procedure TForm1.Button1Click(Sender: TObject);
var
  selstart: integer;
  sellen: integer;
begin
//  showmessage(epGetEdTextAsString(2));
  selstart := epGetEdSelStart(1);
  sellen := epGetEdSelLen(1);
  showmessage(inttostr(selstart)+','+inttostr(sellen));
end;

initialization
  r := TRegExpr.Create;
finalization
  r.free;
end.
